@@ -1,12 +1,14 @@
 # -*- coding: utf-8 -*-
 import base64
 import hashlib
+import json
 
 import six
 import xml
 
-from ks3.bucketCrossReplicate import BucketCrossReplicate
-from ks3.bucketLifecycle import BucketLifecycle
+from ks3.xmlParsers.bucketCors import BucketCors
+from ks3.xmlParsers.bucketCrossReplicate import BucketCrossReplicate
+from ks3.xmlParsers.bucketLifecycle import BucketLifecycle
 from ks3.objectTagging import ObjectTagging
 
 try:
@@ -21,14 +23,14 @@ from ks3.acl import Policy, CannedACLStrings
 from ks3.bucketlistresultset import BucketListResultSet
 from ks3.bucketlistresultset import MultiPartUploadListResultSet
 from ks3.bucketlistresultset import VersionedBucketListResultSet
-from ks3.bucketlogging import BucketLogging
+from ks3.xmlParsers.bucketLogging import BucketLogging
 from ks3.exception import S3ResponseError
 from ks3.key import Key
 from ks3.multipart import MultiPartUpload, CompleteMultiPartUpload
 from ks3.prefix import Prefix
 from ks3.resultset import ResultSet
-from ks3.bucketQuota import BucketQuota
-from ks3.bucketVersioning import BucketVersioningConfig
+from ks3.xmlParsers.bucketQuota import BucketQuota
+from ks3.xmlParsers.bucketVersioning import BucketVersioningConfig
 from ks3.deletemarker import DeleteMarker
 
 try:
@@ -464,12 +466,12 @@ class Bucket(object):
                                  grants=grants)
         return self.set_bucket_logging(blogging.to_xml(), headers=headers)
 
-    def set_bucket_logging(self, logging_str, headers=None):
+    def set_bucket_logging(self, logging_xml, headers=None):
         """
         Set logging on a bucket directly to the given xml string.
 
-        :type logging_str: unicode string
-        :param logging_str: The XML for the bucketloggingstatus which
+        :type logging_xml: unicode string
+        :param logging_xml: The XML for the bucketloggingstatus which
             will be set.  The string will be converted to utf-8 before
             it is sent.  Usually, you will obtain this XML from the
             BucketLogging object.
@@ -480,7 +482,7 @@ class Bucket(object):
 
         if headers is None:
             headers = {}
-        body = logging_str
+        body = logging_xml
         if not isinstance(body, bytes):
             body = body.encode('utf-8')
             # md5 = ks3.utils.compute_base64_md5_digest(body)
@@ -508,7 +510,7 @@ class Bucket(object):
         """
         Get the logging for this bucket.
 
-        :rtype: :class:`ks3.bucketlogging.BucketLogging`
+        :rtype: :class:`ks3.xmlParsers.bucketLogging.BucketLogging`
         :return: A BucketLogging object for this bucket.
         """
         response = self.connection.make_request('GET', self.name,
@@ -793,8 +795,8 @@ class Bucket(object):
         else:
             raise S3ResponseError(response.status, response.reason, body)
 
-    def set_bucket_crr(self, targetBucket, deleteMarkerStatus=None, prefix=None, region=None, headers=None):
-        replicate = BucketCrossReplicate(targetBucket, deleteMarkerStatus, prefix, region)
+    def set_bucket_crr(self, targetBucket, deleteMarkerStatus=None, prefix=None, headers=None):
+        replicate = BucketCrossReplicate(targetBucket, deleteMarkerStatus, prefix)
         if headers is None:
             headers = {}
         headers['Content-Type'] = 'text/xml'
@@ -834,6 +836,7 @@ class Bucket(object):
         else:
             raise S3ResponseError(response.status, response.reason, body)
 
+    # deprecated. moved to key.py
     def restore_object(self, object_key_name, headers=None):
         response = self.connection.make_request('POST', self.name, object_key_name,
                                                 query_args='restore', headers=headers)
@@ -894,6 +897,7 @@ class Bucket(object):
         else:
             raise S3ResponseError(response.status, response.reason, body)
 
+    # deprecated. moved to key.py
     def fetch_object(self, object_key_name, source_url=None, callback_url=None, headers=None):
         if headers is None:
             headers = {}
@@ -983,6 +987,79 @@ class Bucket(object):
         else:
             raise S3ResponseError(response.status, response.reason, body)
 
+    def get_bucket_cors(self):
+        response = self.connection.make_request('GET', self.name,
+                                                query_args='cors')
+        body = response.read()
+        if response.status == 200:
+            bucketCors = BucketCors()
+            h = handler.XmlHandler(bucketCors, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return bucketCors
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_bucket_cors(self, bucket_cors):
+        headers = {'content-type': 'application/xml'}
+        bucket_cors_xml = bucket_cors.to_xml()
+        if not isinstance(bucket_cors_xml, bytes):
+            bucket_cors_xml = bucket_cors_xml.encode('utf-8')
+        md5 = ks3.utils.compute_base64_md5_digest(bucket_cors_xml)
+        headers['content-md5'] = md5
+        response = self.connection.make_request('PUT', self.name, data=bucket_cors_xml,
+                                                query_args='cors',
+                                                headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def delete_bucket_cors(self):
+        response = self.connection.make_request('DELETE', self.name, query_args='cors')
+        body = response.read()
+        if response.status == 204:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_bucket_mirror(self):
+        response = self.connection.make_request('GET', self.name, query_args='mirror')
+        body = response.read()
+        if response.status == 200:
+            return body
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def set_bucket_mirror(self, mirror, headers=None):
+        headers = {'content-type': 'application/json'}
+        if headers is None:
+            headers = {}
+        mirror_dict = dict(mirror)
+        mirror_json = json.dumps(mirror_dict)
+        if not isinstance(mirror_json, bytes):
+            mirror_json = mirror_json.encode('utf-8')
+        md5 = ks3.utils.compute_base64_md5_digest(mirror_json)
+        headers['content-md5'] = md5
+        response = self.connection.make_request('PUT', self.name, data=mirror_json,
+                                                query_args='mirror',
+                                                headers=headers)
+        body = response.read()
+        if response.status == 200:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def delete_bucket_mirror(self):
+        response = self.connection.make_request('DELETE', self.name, query_args='mirror')
+        body = response.read()
+        if response.status == 204:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
 
 class BucketLocation(object):
     def __init__(self):
