@@ -6,7 +6,12 @@ import math
 import mimetypes
 import os
 import re
+import xml
 from hashlib import md5
+
+from ks3 import handler
+from ks3.objectTagging import ObjectTagging
+
 try:
     import urllib.parse as parse
 except ImportError:
@@ -14,7 +19,7 @@ except ImportError:
 import binascii
 
 import errno
-from ks3.exception import KS3ClientError
+from ks3.exception import KS3ClientError, S3ResponseError
 from ks3.exception import StorageDataError, PleaseRetryException
 from ks3.keyfile import KeyFile
 from ks3.user import User
@@ -946,7 +951,62 @@ class Key(object):
         if encoding is not None:
             value = value.decode(encoding)
         return value
-        
+
+    def restore_object(self, headers=None):
+        self.bucket.restore_object(self.name, headers=headers)
+
+    def set_object_tagging(self, tagging_set, version_id=None, headers=None):
+        object_tagging = ObjectTagging(tagging_set)
+        if headers is None:
+            headers = {}
+        headers['Content-Type'] = 'text/xml'
+        object_tagging_xml = object_tagging.to_xml()
+        if not isinstance(object_tagging_xml, bytes):
+            object_tagging_xml = object_tagging_xml.encode('utf-8')
+
+        query_args = 'tagging'
+        if version_id is not None:
+            query_args = query_args + '&versionId=' + version_id
+        response = self.bucket.connection.make_request('PUT', self.bucket.name, self.name, data=object_tagging_xml,
+                                                query_args=query_args, headers=headers)
+
+        body = response.read()
+        if response.status == 200:
+            return True
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def get_object_tagging(self, version_id=None, headers=None):
+        query_args = 'tagging'
+        if version_id is not None:
+            query_args = query_args + '&versionId=' + version_id
+        response = self.bucket.connection.make_request('GET', self.bucket.name, self.name,
+                                                query_args=query_args, headers=headers)
+        body = response.read()
+        if response.status == 200:
+            objectTagging = ObjectTagging()
+            h = handler.XmlHandler(objectTagging, self)
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
+            xml.sax.parseString(body, h)
+            return objectTagging
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def delete_object_tagging(self, version_id=None, headers=None):
+        query_args = 'tagging'
+        if version_id is not None:
+            query_args = query_args + '&versionId=' + version_id
+        response = self.bucket.connection.make_request('DELETE', self.bucket.name, self.name,
+                                                query_args=query_args, headers=headers)
+        body = response.read()
+        if response.status == 204:
+            return None
+        else:
+            raise S3ResponseError(response.status, response.reason, body)
+
+    def fetch_object(self, source_url=None, callback_url=None, headers=None):
+        self.bucket.fetch_object(self.name, source_url=source_url, callback_url=callback_url, headers=headers)
 
     def handle_version_headers(self, resp, force=False):
         provider = self.bucket.connection.provider
